@@ -52,13 +52,20 @@ static KMenu* buildMenuForColor(const QColor &color)
     act->setData(colorData);
     act = menu->addAction(mimeIcon, htmlName.mid(1));
     act->setData(colorData);
-    if (htmlNameUp != htmlName)
-    {
+
+    if (htmlNameUp != htmlName) {
         act = menu->addAction(mimeIcon, htmlNameUp);
         act->setData(colorData);
         act = menu->addAction(mimeIcon, htmlNameUp.mid(1));
         act->setData(colorData);
     }
+
+    menu->addSeparator();
+    act = menu->addAction(mimeIcon, "Latex Color");
+    act->setData(colorData);
+
+    act = menu->addAction(mimeIcon, htmlName + QString::fromLatin1("ff"));
+    act->setData(colorData);
     return menu;
 }
 
@@ -87,7 +94,6 @@ static QColor pickColor(const QPoint &point)
 #endif
 }
 
-
 class ColorIconEngine : public QIconEngine
 {
     public:
@@ -130,7 +136,6 @@ QPixmap ColorIconEngine::pixmap(const QSize &size, QIcon::Mode mode, QIcon::Stat
     return pix;
 }
 
-
 class ColorIcon : public QIcon
 {
     public:
@@ -153,6 +158,9 @@ Kolourpicker::Kolourpicker(QObject *parent, const QVariantList &args)
     mainlay->setSpacing(4);
     mainlay->setContentsMargins(0.0, 0.0, 0.0, 0.0);
 
+    m_grabWidget = new QWidget( 0,  Qt::X11BypassWindowManagerHint );
+    m_grabWidget->move( -1000, -1000 );
+
     m_grabButton = new Plasma::ToolButton(this);
     m_grabButton->setMinimumSize(20, 20);
     mainlay->addItem(m_grabButton);
@@ -160,26 +168,54 @@ Kolourpicker::Kolourpicker(QObject *parent, const QVariantList &args)
     m_grabButton->nativeWidget()->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
     connect(m_grabButton, SIGNAL(clicked()), this, SLOT(grabClicked()));
 
-    m_historyButton = new Plasma::ToolButton(this);
-    m_historyButton->setMinimumSize(20, 20);
-    mainlay->addItem(m_historyButton);
-    m_historyButton->setEnabled(false);
-    m_historyButton->nativeWidget()->setIcon(ColorIcon(Qt::gray));
-    m_historyButton->nativeWidget()->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
-    connect(m_historyButton, SIGNAL(clicked()), this, SLOT(historyClicked()));
+    m_configAndHistory = new Plasma::ToolButton(this);
+    m_configAndHistory->setMinimumSize(20, 20);
+    mainlay->addItem(m_configAndHistory);
+
+    m_configAndHistory->nativeWidget()->setIcon(ColorIcon(Qt::gray));
+    m_configAndHistory->nativeWidget()->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    connect(m_configAndHistory, SIGNAL(clicked()), this, SLOT(historyClicked()));
 
     KMenu *menu = new KMenu();
-    menu->addTitle(i18n("History"));
-    m_historyMenu = menu;
-    m_historyMenu->addSeparator();
-    QAction *act = m_historyMenu->addAction(KIcon("edit-clear-history"), i18n("Clear History"));
+    menu->addTitle(i18n("Color Options"));
+    m_configAndHistoryMenu = menu;
+    QAction *act = m_configAndHistoryMenu->addAction(KIcon("edit-clear-history"), i18n("Clear History"));
     connect(act, SIGNAL(triggered(bool)), this, SLOT(clearHistory()));
+    m_configAndHistoryMenu->addSeparator();
+
+    // building the menu for default color string format.
+    KMenu *m_colors_menu = new KMenu();
+    m_colors_menu->addTitle(i18n("Default Format"));
+    m_colors_format << "r, g, b" << "#RRGGBB" << "RRGGBB" << "#rrggbb" << "rrggbb";
+    foreach (const QString& s, m_colors_format) {
+      act = m_colors_menu->addAction(KIcon("draw-text"), s);
+      act->setData(s);
+    }
+
+    m_colors_menu->addSeparator();
+    act = m_colors_menu->addAction(KIcon("draw-text"), "Latex");
+    act->setData("Latex");
+
+    connect(m_colors_menu, SIGNAL(triggered(QAction*)), this, SLOT(setDefaultColorFormat(QAction*)));
+    act = m_colors_menu->menuAction();
+    act->setText(i18n("Default Color Format"));
+    m_configAndHistoryMenu->addMenu(m_colors_menu);
+}
+
+void Kolourpicker::setDefaultColorFormat(QAction *act)
+{
+    if (!act) {
+      return;
+    }
+
+    m_color_format = qvariant_cast<QString>(act->data());
 }
 
 Kolourpicker::~Kolourpicker()
 {
     clearHistory(false);
-    delete m_historyMenu;
+    delete m_grabWidget;
+    delete m_configAndHistoryMenu;
 }
 
 void Kolourpicker::init()
@@ -187,12 +223,14 @@ void Kolourpicker::init()
     KConfigGroup cg = config();
 
     QList<QString> colorList = cg.readEntry("Colors", QList<QString>());
-    Q_FOREACH (const QString &color, colorList)
-    {
+    m_color_format = cg.readEntry("ColorFormat", QString());
+
+    Q_FOREACH (const QString &color, colorList) {
         addColor(QColor(color), false);
     }
 }
 
+
 void Kolourpicker::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & Plasma::FormFactorConstraint) {
@@ -214,51 +252,24 @@ void Kolourpicker::constraintsEvent(Plasma::Constraints constraints)
     }
 }
 
-bool Kolourpicker::sceneEventFilter(QGraphicsItem *watched, QEvent *event)
-{
-    if (watched == m_grabButton && event->type() == QEvent::GraphicsSceneMouseRelease)
-    {
-        m_grabWidget = static_cast<QGraphicsSceneMouseEvent *>(event)->widget();
-        if (m_grabWidget && m_grabWidget->parentWidget())
-        {
-            m_grabWidget = m_grabWidget->parentWidget();
-        }
-        if (m_grabWidget)
-        {
-            m_grabWidget->installEventFilter(this);
-        }
-    }
-    return false;
-}
-
 bool Kolourpicker::eventFilter(QObject *watched, QEvent *event)
 {
-    if (watched == m_grabWidget && event->type() == QEvent::MouseButtonRelease)
-    {
+    if (watched == m_grabWidget && event->type() == QEvent::MouseButtonRelease) {
         m_grabWidget->removeEventFilter(this);
+	m_grabWidget->hide();
         m_grabWidget->releaseMouse();
-
         QMouseEvent *me = static_cast<QMouseEvent *>(event);
-
         const QColor color = pickColor(me->globalPos());
-
         kDebug() << event->type() << me->globalPos() << color;
-
         addColor(color);
-
-        KMenu *newmenu = buildMenuForColor(color);
-        newmenu->addTitle(ColorIcon(color), i18n("Copy Color Value"), newmenu->actions().first());
-        connect(newmenu, SIGNAL(triggered(QAction*)), this, SLOT(colorActionTriggered(QAction*)));
-        newmenu->exec(QCursor::pos());
-        delete newmenu;
+	colorActionTriggered(color);
     }
     return Plasma::Applet::eventFilter(watched, event);
 }
 
 QVariant Kolourpicker::itemChange(GraphicsItemChange change, const QVariant &value)
 {
-    if (change == ItemSceneChange)
-    {
+    if (change == ItemSceneChange) {
         QMetaObject::invokeMethod(this, "installFilter", Qt::QueuedConnection);
     }
     return Plasma::Applet::itemChange(change, value);
@@ -266,47 +277,91 @@ QVariant Kolourpicker::itemChange(GraphicsItemChange change, const QVariant &val
 
 void Kolourpicker::grabClicked()
 {
-    if (m_grabWidget)
-    {
+    if (m_grabWidget) {
+	m_grabWidget->show();
+	m_grabWidget->installEventFilter( this );
         m_grabWidget->grabMouse(Qt::CrossCursor);
     }
 }
 
 void Kolourpicker::historyClicked()
 {
-    m_historyMenu->popup(QCursor::pos());
+    m_configAndHistoryMenu->popup(QCursor::pos());
+}
+
+void Kolourpicker::colorActionTriggered(const QColor& color)
+{
+    if (!color.isValid()) {
+        return;
+    }
+
+    QMimeData *mime = new QMimeData();
+    mime->setColorData(color);
+
+    QString text;
+    /*
+    converts the color according to the color format choosen by the user
+    */
+    if (m_color_format == "r, g, b") {
+       text = QString("%1, %2, %3").arg(color.red()).arg(color.green()).arg(color.blue());
+    } else if(m_color_format == "#RRGGBB") {
+      text = color.name().toUpper();
+    } else if(m_color_format == "RRGGBB") {
+      text = color.name().toUpper().mid(1);
+    } else if(m_color_format == "#rrggbb") {
+      text = color.name();
+    } else if(m_color_format == "rrggbb") {
+      text = color.name().mid(1);
+    } else if(m_color_format == "Latex") {
+      text = toLatex(color);
+    } else {
+      text = QString("%1, %2, %3").arg(color.red()).arg(color.green()).arg(color.blue());
+    }
+
+    mime->setText(text);
+    QApplication::clipboard()->setMimeData(mime, QClipboard::Clipboard);
+}
+
+QString Kolourpicker::toLatex(const QColor& color)
+{
+      qreal r = (qreal)color.red()/255;
+      qreal g = (qreal)color.green()/255;
+      qreal b = (qreal)color.blue()/255;
+
+      return QString("\\definecolor{ColorName}{rgb}{%1,%2,%3}").arg(r,0,'f',2).arg(g,0,'f',2).arg(b,0,'f',2);
 }
 
 void Kolourpicker::colorActionTriggered(QAction *act)
 {
-    if (!act)
+    if (!act) {
         return;
+    }
 
     QColor color = qvariant_cast<QColor>(act->data());
-    if (!color.isValid())
-        return;
+    QString text =  act->text().remove('&');
+
+    if(text == i18n("Latex Color")) {
+      text = toLatex(color);
+    }
 
     QMimeData *mime = new QMimeData();
     mime->setColorData(color);
-    mime->setText(act->text().remove(QChar('&')));
+    mime->setText(text);
     QApplication::clipboard()->setMimeData(mime, QClipboard::Clipboard);
 }
 
 void Kolourpicker::clearHistory(bool save)
 {
-    m_historyButton->setEnabled(false);
-    m_historyButton->nativeWidget()->setIcon(ColorIcon(Qt::gray));
+    m_configAndHistory->nativeWidget()->setIcon(ColorIcon(Qt::gray));
     QHash<QColor, QAction *>::ConstIterator it = m_menus.constBegin(), itEnd = m_menus.constEnd();
-    for ( ; it != itEnd; ++it )
-    {
-        m_historyMenu->removeAction(*it);
+    for (; it != itEnd; ++it ) {
+        m_configAndHistoryMenu->removeAction(*it);
         delete *it;
     }
     m_menus.clear();
     m_colors.clear();
 
-    if (save)
-    {
+    if (save) {
         KConfigGroup cg = config();
         saveData(cg);
     }
@@ -320,8 +375,9 @@ void Kolourpicker::installFilter()
 void Kolourpicker::addColor(const QColor &color, bool save)
 {
     QHash<QColor, QAction *>::ConstIterator it = m_menus.constFind(color);
-    if (it != m_menus.constEnd())
+    if (it != m_menus.constEnd()) {
         return;
+    }
 
     KMenu *newmenu = buildMenuForColor(color);
     QAction *act = newmenu->menuAction();
@@ -329,13 +385,12 @@ void Kolourpicker::addColor(const QColor &color, bool save)
     act->setIcon(colorIcon);
     act->setText(QString("%1, %2, %3").arg(color.red()).arg(color.green()).arg(color.blue()));
     connect(newmenu, SIGNAL(triggered(QAction*)), this, SLOT(colorActionTriggered(QAction*)));
-    m_historyMenu->insertMenu(m_historyMenu->actions().at(1), newmenu);
-    m_historyButton->nativeWidget()->setIcon(colorIcon);
+    m_configAndHistoryMenu->insertMenu(m_configAndHistoryMenu->actions().at(1), newmenu);
+    m_configAndHistory->nativeWidget()->setIcon(colorIcon);
     m_menus.insert(color, act);
     m_colors.append(color.name());
-    m_historyButton->setEnabled(true);
-    if (save)
-    {
+    m_configAndHistory->setEnabled(true);
+    if (save) {
         KConfigGroup cg = config();
         saveData(cg);
     }
@@ -344,7 +399,7 @@ void Kolourpicker::addColor(const QColor &color, bool save)
 void Kolourpicker::saveData(KConfigGroup &cg)
 {
     cg.writeEntry("Colors", m_colors);
-
+    cg.writeEntry("ColorFormat", m_color_format);
     emit configNeedsSaving();
 }
 
