@@ -35,13 +35,21 @@
 #include <QClipboard>
 #include <QTimer>
 #include <QPen>
+#include <QWeakPointer>
 
 class QSignalMapper;
 class KAction;
+class QPropertyAnimation;
+
+namespace Plasma
+{
+    class Service;
+} // namespace Plasma
 
 class Pastebin : public Plasma::Applet
 {
     Q_OBJECT
+    Q_PROPERTY(qreal animationUpdate READ animationValue WRITE animationUpdate)
 public:
     Pastebin(QObject *parent, const QVariantList &args);
     ~Pastebin();
@@ -54,6 +62,8 @@ public:
                         const QRect &contents);
     void constraintsEvent(Plasma::Constraints constraints);
 
+    qreal animationValue() const;
+
     enum InteractionState { /* What is the user doing, used for visual feedback on user actions */
         Off = 0,            /* Not set */
         Waiting = 1,        /* Applet hanging around idle */
@@ -71,7 +81,7 @@ public:
     };
 
     enum textServers { PASTEBINCA, PASTEBINCOM };
-    enum imageServers { IMAGEBINCA, IMAGESHACK, SIMPLESTIMAGEHOSTING };
+    enum imageServers { IMAGEBINCA, IMAGESHACK, SIMPLESTIMAGEHOSTING, IMGUR };
 
 public slots:
     void configAccepted();
@@ -82,7 +92,7 @@ protected slots:
     virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
     void showResults(const QString &url);
     void showErrors();
-    void openLink();
+    void openLink(bool old = true);
     void postClipboard();
 
 protected:
@@ -101,16 +111,19 @@ private slots:
     void updateTheme();
     void resetActionState();
     void copyToClipboard(const QString &url);
+    void postingFinished(Plasma::ServiceJob *job);
 
     void editTextServer();
     void saveTextServer();
     void editImageServer();
     void saveImageServer();
     void closeServerDialog();
+
 private:
     int iconSize();
     void showOverlay(bool show);
     void postContent(QString text, QImage imageData);
+    void postClipboard(bool preferSelection);
 
     void addToHistory(const QString &url);
 
@@ -122,24 +135,24 @@ private:
 
     bool m_isHovered;
     bool m_fadeIn;
-    int m_animId;
+    QWeakPointer<QPropertyAnimation> m_animation;
     qreal m_alpha;
 
     QFont m_font;
     QPen m_linePen;
     QColor m_fgColor;
     QColor m_bgColor;
-    KIcon* m_icon;
 
     Plasma::ToolTipContent toolTipData;
     QString m_url;
+    QString m_oldUrl;
 
     int m_textBackend;
     int m_imageBackend;
     int m_imagePrivacy;
     QString m_server;
 
-    QTimer *timer;
+    QTimer *m_timer;
 
     int m_historySize;
     QSignalMapper *m_signalMapper;
@@ -151,12 +164,15 @@ private:
 
     // custom server dialog
     QWidget *m_servers;
-    bool m_waiting;
 
     QClipboard::Mode lastMode;
 
     Ui::pastebinConfig uiConfig;
     Ui::pastebinServersConfig uiServers;
+
+    Plasma::Service *m_postingService;
+
+    QString m_hoverText;
 };
 
 K_EXPORT_PLASMA_APPLET(pastebin, Pastebin)
