@@ -23,16 +23,15 @@
 #include <QGraphicsSceneWheelEvent>
 #include <QPainter>
 #include <QStyleOptionGraphicsItem>
-#include <QTimeLine>
+#include <QPropertyAnimation>
 
 // KDE
 #include <KDebug>
-#include <KConfigGroup>
 #include <KGlobalSettings>
-#include <KService>
 #include <KIconLoader>
 #include <KIO/PreviewJob>
 #include <KUrl>
+#include <KIcon>
 
 // Plasma
 #include <Plasma/Animator>
@@ -41,14 +40,15 @@
 #include <Plasma/Svg>
 #include <Plasma/Theme>
 
-const int EXPANDING_DURATION = 200;
+static const int EXPANDING_DURATION = 200;
+static const int REMOVE_EMBLEM_SIZE = 22;
 
 PreviewWidget::PreviewWidget(QGraphicsItem *parent)
     : QGraphicsWidget(parent),
       m_selectedIndex(-1),
       m_hoveredIndex(-1),
-      m_closeStatus(true),
-      m_animId(-1)
+      m_hoverSvg(new Plasma::FrameSvg(this)),
+      m_closeStatus(true)
 {
     m_scrollBar = new Plasma::ScrollBar(this);
     connect(m_scrollBar, SIGNAL(valueChanged(int)), this, SLOT(scrolled( int )));
@@ -63,13 +63,21 @@ PreviewWidget::PreviewWidget(QGraphicsItem *parent)
 
     // let's draw the top border of the applet
     m_panel = new Plasma::FrameSvg(this);
-    m_panel->setImagePath("dialogs/background");
+    m_panel->setImagePath("widgets/background");
 
     m_logo = new Plasma::Svg(this);
     m_logo->setImagePath("widgets/previewer-16");
 
+    m_hoverSvg->setImagePath("widgets/viewitem");
+    m_hoverSvg->setEnabledBorders(Plasma::FrameSvg::AllBorders);
+    m_hoverSvg->setCacheAllRenderedFrames(true);
+
+    updateMargins();
+
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
             this, SLOT(setupOptionViewItem()));
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
+            this, SLOT(updateMargins()));
 }
 
 PreviewWidget::~PreviewWidget()
@@ -116,7 +124,7 @@ void PreviewWidget::layoutItems()
     m_scrollBar->setSingleStep(itemHeight + s_spacing);
     m_scrollBar->setPageStep(m_itemsRect.height());
 
-    if (m_animId < 1) {
+    if (m_animation.isNull() || m_animation.data()->state() != QAbstractAnimation::Running) {
         if (m_items.last().bottom() > m_itemsRect.bottom() || m_items.first().top() < m_itemsRect.top()) {
             if (!m_scrollBar->isVisible()) {
                 m_scrollBar->show();
@@ -149,7 +157,6 @@ void PreviewWidget::wheelEvent(QGraphicsSceneWheelEvent *event)
 
 void PreviewWidget::lookForPreview()
 {
-    m_previewReady = false;
     if (m_previewHistory.isEmpty()) {
         return;
     }
@@ -168,18 +175,7 @@ void PreviewWidget::lookForPreview()
 void PreviewWidget::setPreview(const KFileItem &item, const QPixmap &pixmap)
 {
     m_previews.insert(item.url(), pixmap);
-    m_previewReady = true;
-    updatePreview();
-}
-
-void PreviewWidget::updatePreview()
-{
-    //NOTE: use this to update the exact rect in order to repaint
-    //      correctly the preview
-
-    QRect rect = m_previewRect;
-    rect.setHeight(rect.height() + bottomBorderHeight());
-    update(rect);
+    update();
 }
 
 void PreviewWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
@@ -211,11 +207,25 @@ void PreviewWidget::updateSelectedItems(const QPoint &point)
     for (int i = 0; i < m_items.count(); i++) {
         if (m_items[i].contains(point)) {
             m_selectedIndex = i;
-            emit fileOpenRequested(KUrl(m_previewHistory[i]));
             break;
         }
     }
 
+    if (m_selectedIndex == -1) {
+        return;
+    }
+
+    // here we check if the remove emblem was clicked
+    QRect r = m_items[m_selectedIndex];
+    r.setX(r.right() - REMOVE_EMBLEM_SIZE);
+    r.setSize(QSize(REMOVE_EMBLEM_SIZE, REMOVE_EMBLEM_SIZE));
+    if (r.contains(point)) {
+        removeItem(m_selectedIndex);
+        return;
+    }
+
+    emit fileOpenRequested(KUrl(m_previewHistory[m_selectedIndex]));
+
     if (m_selectedIndex != previous) {
         if (m_selectedIndex != -1) {
             update(m_items[m_selectedIndex]);
@@ -229,6 +239,12 @@ void PreviewWidget::updateSelectedItems(const QPoint &point)
 
 void PreviewWidget::updateHoveredItems(const QPoint &point)
 {
+
+    // updating is unsafe when layout is broken
+    if (!m_layoutIsValid) {
+        return;
+    }
+
     m_hoveredUrl = KUrl();
     const int previous = m_hoveredIndex;
     m_hoveredIndex = -1;
@@ -243,7 +259,7 @@ void PreviewWidget::updateHoveredItems(const QPoint &point)
                 KUrl item = KUrl(m_previewHistory[i]);
                 if (m_previews.contains(item)) {
                     m_hoveredUrl = item;
-                    updatePreview();
+                    update();
                 }
 
                 break;
@@ -279,14 +295,8 @@ void PreviewWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 
         if (m_itemsRect.contains(event->pos().toPoint())) {
             updateSelectedItems(event->pos().toPoint());
-        }
-
-        if (m_arrowRect.contains(event->pos().toPoint())) {
-            if (m_closeStatus) {
-                expand();
-            } else {
-                contract();
-            }
+        } else if (m_arrowRect.contains(event->pos().toPoint())) {
+            animateHeight(m_closeStatus);
         }
     }
 
@@ -308,7 +318,16 @@ void PreviewWidget::setItemsList(const QList<QUrl> &list)
     m_previewHistory.clear();
     m_previewHistory = list;
 
-    calculateRects();
+    m_layoutIsValid = false;
+    update();
+
+    lookForPreview();
+}
+
+void PreviewWidget::addItem(const QUrl &url)
+{
+    m_previewHistory << url;
+    m_layoutIsValid = false;
     update();
 
     lookForPreview();
@@ -322,7 +341,7 @@ void PreviewWidget::resizeEvent(QGraphicsSceneResizeEvent *event)
 
 void PreviewWidget::calculateRects()
 {
-    QRect rect = contentsRect().toRect();
+    QRect rect = this->rect().toRect();
     const int scrollBarWidth = m_scrollBar->isVisible() ? int(m_scrollBar->preferredSize().width()) : 0;
     const int itemWidth = iconSize().width() * 2;
     const int itemRectWidth = itemWidth + scrollBarWidth + (s_spacing * 2) +
@@ -330,24 +349,26 @@ void PreviewWidget::calculateRects()
     const int bottomBorder = bottomBorderHeight();
 
     // items rect shouldn't be visible even if the status is close
-    if (m_previewHistory.isEmpty() || (m_closeStatus && m_animId < 1)) {
+    const bool isAnimating = m_animation.isNull() || m_animation.data()->state() != QAbstractAnimation::Running;
+    if (m_previewHistory.isEmpty() || (m_closeStatus && isAnimating)) {
         m_animationHeight = s_topBorder + bottomBorder;
-        m_scrollBar->setGeometry(QRect());
-    } else {
-        if (m_animId < 1) {
-            m_animationHeight = rect.height();
-        }
-        const int itemRectHeight = m_animationHeight - s_topBorder - bottomBorder;
-        m_itemsRect = QRect(5, s_topBorder, itemRectWidth, itemRectHeight);
+    } else if (isAnimating) {
+        m_animationHeight = rect.height();
+    }
 
+    const int itemRectHeight = m_animationHeight - s_topBorder - bottomBorder;
+    m_itemsRect = QRect(contentsRect().x(), s_topBorder, itemRectWidth, itemRectHeight);
+
+    if (itemRectHeight) {
         QRect r = QRect(m_itemsRect.right() - scrollBarWidth + 2, s_topBorder + 1, scrollBarWidth,
                         m_animationHeight - s_topBorder - bottomBorder - 2);
         m_scrollBar->setGeometry(r);
+    } else {
+        m_scrollBar->setGeometry(QRect());
     }
 
     //kDebug() << m_animationHeight;
-
-    m_arrowRect = QRect(rect.width() - 30 - 5, (s_topBorder / 2), 10,10);
+    m_arrowRect = QRect(contentsRect().right() - 10, (s_topBorder / 2), 10,10);
 
     m_previewRect = QRect(m_itemsRect.right(), m_itemsRect.y(),
                          rect.width() - m_itemsRect.right() - rect.x(),
@@ -356,77 +377,56 @@ void PreviewWidget::calculateRects()
     m_layoutIsValid = false;
 }
 
+qreal PreviewWidget::animationValue() const
+{
+    return m_animationHeight;
+}
+
 void PreviewWidget::expandingSlot(qreal progress)
 {
     const int min = s_topBorder + bottomBorderHeight();
-    if (qFuzzyCompare(qreal(1.0), progress)) {
-        m_animId = -1;
 
-        if (m_closeStatus) {
-            m_animationHeight = min;
-        } else {
-            m_animationHeight = int(size().height()) - min;
-
-            // show the scroll bar again if necessary
-            if (!m_items.isEmpty() && (m_items.last().bottom() > m_itemsRect.bottom() ||
-                m_items.first().top() < m_itemsRect.top())) {
-                m_scrollBar->show();
-            }
-        }
-    } else {
-        m_animationHeight = qMax(qreal(min),
-                                 min - 1 + ((size().height() - min)  * 
-                                            (m_closeStatus ? qreal(1.0) - progress : progress)));
+    if (!m_closeStatus && !m_items.isEmpty() && (m_items.last().bottom() > m_itemsRect.bottom() ||
+        m_items.first().top() < m_itemsRect.top())) {
+        m_scrollBar->show();
     }
 
+    m_animationHeight = qMax(qreal(min), min - 1 + ((size().height() - min)  * progress));
+
     calculateRects();
     update();
 }
 
-void PreviewWidget::contract()
+void PreviewWidget::animateHeight(bool isExpanding)
 {
-    if (m_closeStatus) {
+    if (m_closeStatus != isExpanding) {
         return;
     }
 
-    m_closeStatus = true;
+    m_closeStatus = !isExpanding;
 
-    if (m_animId > 0) {
-        Plasma::Animator::self()->stopCustomAnimation(m_animId);
+    QPropertyAnimation *animation = m_animation.data();
+    if (!animation) {
+        animation = new QPropertyAnimation(this, "animationValue");
+        animation->setDuration(EXPANDING_DURATION);
+        animation->setStartValue(0.0);
+        animation->setEndValue(1.0);
+        animation->setEasingCurve(QEasingCurve::InOutQuad);
+        m_animation = animation;
+    } else if (animation->state() == QAbstractAnimation::Running) {
+        animation->pause();
     }
 
     // don't draw the scrollbar while animating
     m_scrollBar->hide();
 
-    // nasty hack alert: expandingSlot will get called before customAnimation returns
-    // and we rely on m_animId != -1 .. so .. set it to a dummy value here
-    m_animId = 1;
-    m_animId = Plasma::Animator::self()->customAnimation(20, EXPANDING_DURATION,
-                                                         Plasma::Animator::EaseInOutCurve,
-                                                         this, "expandingSlot");
-}
-
-void PreviewWidget::expand()
-{
-    if (!m_closeStatus) {
-        return;
+    if (isExpanding) {
+        animation->setDirection(QAbstractAnimation::Forward);
+        animation->start(QAbstractAnimation::KeepWhenStopped);
+    } else {
+        animation->setDirection(QAbstractAnimation::Backward);
+        animation->start(QAbstractAnimation::DeleteWhenStopped);
     }
-
-    m_closeStatus = false;
-
-    if (m_animId > 0) {
-        Plasma::Animator::self()->stopCustomAnimation(m_animId);
-    }
-
-    // don't draw the scrollbar while animating
-    m_scrollBar->hide();
-
-    // nasty hack alert: expandingSlot will get called before customAnimation returns
-    // and we rely on m_animId != -1 .. so .. set it to a dummy value here
-    m_animId = 1;
-    m_animId = Plasma::Animator::self()->customAnimation(20, EXPANDING_DURATION,
-                                                         Plasma::Animator::EaseInOutCurve,
-                                                         this, "expandingSlot");
 }
 
 void PreviewWidget::scrolled( int value )
@@ -466,8 +466,8 @@ void PreviewWidget::paint(QPainter *painter,
     }
 
     const int bottomBorder = bottomBorderHeight();
-    QRect contentsRect = this->contentsRect().toRect();
-    painter->translate(contentsRect.topLeft());
+    QRect contentsRect = opt->rect;
+//    painter->translate(contentsRect.topLeft());
     painter->setRenderHint(QPainter::Antialiasing);
 
     painter->save();
@@ -488,27 +488,27 @@ void PreviewWidget::paint(QPainter *painter,
 
     m_logo->paint(painter, QRect(10, 5, s_logoSize, s_logoSize));
 
+    // FIXME: probably too much magic numbers
     painter->save();
     QFont font = KGlobalSettings::smallestReadableFont();
     font.setBold(true);
     painter->setFont(font);
     painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
     painter->drawText(QRect(39, 3, contentsRect.width() - 39, 30), Qt::AlignBottom | Qt::AlignLeft,
-                      " " + i18n("Previewer"));
-    QFontMetrics fm(font);
-    int length = fm.width(i18n("Previewer") + "  ");
+                      ' ' + i18n("Previewer"));
+    int length = opt->fontMetrics.width(i18n("Previewer") + "  ");
     font.setBold(false);
     painter->setFont(KGlobalSettings::smallestReadableFont());
-    painter->drawText(QRect(39 + length, 3, contentsRect.width() - 39, 30),
+    QString elidedHint = opt->fontMetrics.elidedText(i18n("Drop files on me to preview them."), Qt::ElideRight, contentsRect.width() - 39 - length - 3);
+    painter->drawText(QRect(39 + length + 3, 3, contentsRect.width() - 39 - length - 3, 30),
                       Qt::AlignBottom | Qt::AlignLeft,
-                      i18n("Drop files on me to preview them."));
+                      elidedHint);
     painter->restore();
 
     drawOpenCloseArrow(painter);
 
     if (!m_items.isEmpty() && m_itemsRect.isValid()) {
         painter->save();
-        painter->setClipping(true);
         painter->setClipRect(m_itemsRect);
 
         PreviewItemModel model(m_previewHistory);
@@ -525,13 +525,20 @@ void PreviewWidget::paint(QPainter *painter,
             }
 
             m_option.state &= ~(QStyle::State_Selected | QStyle::State_MouseOver) ;
+	    m_hoverSvg->setElementPrefix("");
             if (m_selectedIndex == i) {
-                m_option.state |= QStyle::State_Selected;
+                m_hoverSvg->prefix().isEmpty() ? m_hoverSvg->setElementPrefix("selected") : m_hoverSvg->setElementPrefix(m_hoverSvg->prefix() + "+selected");
             }
 
             if (m_hoveredIndex == i) {
-                m_option.state |= QStyle::State_MouseOver;
+                m_hoverSvg->prefix().isEmpty() ? m_hoverSvg->setElementPrefix("hover") : m_hoverSvg->setElementPrefix(m_hoverSvg->prefix() + "+hover");
             }
+	    
+	    if (!m_hoverSvg->prefix().isEmpty()) {
+	        m_hoverSvg->resizeFrame(r.size());
+                m_hoverSvg->paintFrame(painter, r.topLeft());
+                KIcon("list-remove").paint(painter, r.right() - REMOVE_EMBLEM_SIZE, r.y(), REMOVE_EMBLEM_SIZE, REMOVE_EMBLEM_SIZE);
+	    }
 
             m_option.rect = r;
             m_delegate.setShadowColor(Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
@@ -543,16 +550,9 @@ void PreviewWidget::paint(QPainter *painter,
     }
 
     // hovering rect
-    if (m_previewReady && !m_hoveredUrl.isEmpty()) {
+    if (!m_hoveredUrl.isEmpty() && m_previews.contains(m_hoveredUrl)) {
         QPixmap pmap;
-        if (m_previews.contains(m_hoveredUrl)) {
-            pmap = m_previews[m_hoveredUrl];
-            QRect itemNameRect = QRect(m_itemsRect.right(), m_itemsRect.bottom(),
-                                       contentsRect.width() - m_itemsRect.right(), bottomBorder);
-            // here we paint the file name
-            painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
-            painter->drawText(itemNameRect, Qt::AlignVCenter | Qt::AlignLeft, m_hoveredUrl.fileName());
-        }
+        pmap = m_previews[m_hoveredUrl];
 
         if (m_previewRect.width() >= pmap.width() && m_previewRect.height() >= pmap.height()) {
             QRect p_rect = QRect(m_previewRect.x() + abs(m_previewRect.width() - pmap.width())/2,
@@ -621,3 +621,36 @@ int PreviewWidget::suggestedWidth()
 
     return (39 + length + fm.width(i18n("Drop files on me to preview them.")) + 70);
 }
+
+QList<QUrl> PreviewWidget::previews()
+{
+    return m_previewHistory;
+}
+
+void PreviewWidget::removeItem(int index)
+{
+    m_previewHistory.removeAt(index);
+    m_selectedIndex = -1;
+    m_hoveredIndex = -1;
+    m_layoutIsValid = false;
+    if (m_previewHistory.isEmpty()) {
+        animateHeight(false);
+    }
+    lookForPreview();
+    update();
+}
+
+void PreviewWidget::updateMargins()
+{
+    if (!m_panel) {
+        return;
+    }
+
+    qreal left;
+    qreal top;
+    qreal right;
+    qreal bottom;
+
+    m_panel->getMargins(left, top, right, bottom);
+    setContentsMargins(left, top, right, bottom);
+}
