@@ -26,6 +26,7 @@
 #include <QApplication>
 #include <QGraphicsLinearLayout>
 #include <QGraphicsGridLayout>
+#include <QPainter>
 #include <QStandardItemModel>
 
 #include <KGlobalSettings>
@@ -50,14 +51,59 @@ template <typename T> T clampValue(T value, int decimals)
         const T mul = std::pow(static_cast<T>(10), decimals); return int(value * mul) / mul;
 }
 
+bool isValidIconName(const QString &icon)
+{
+    return !icon.isEmpty() &&
+           !KIconLoader::global()->loadIcon(icon, KIconLoader::Desktop, 0,
+                                            KIconLoader::DefaultState, QStringList(), 0, true).isNull();
+}
+
+class BackgroundWidget : public QGraphicsWidget
+{
+public:
+    BackgroundWidget(QGraphicsWidget *parent)
+        : QGraphicsWidget(parent)
+    {
+    }
+
+    KIcon currentWeather() const
+    {
+        return m_currentWeather;
+    }
+
+    void setCurrentWeather(const QString &currentWeather = QString())
+    {
+        kDebug() << "!!!!!!!!!!!!!!!!!!!!!!!" << currentWeather;
+        if (isValidIconName(currentWeather)) {
+            m_currentWeather = KIcon(currentWeather);
+        } else {
+            m_currentWeather = KIcon();
+        }
+
+        update();
+    }
+
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
+    {
+        if (m_currentWeather.isNull()) {
+            return;
+        }
+
+        QSize s(KIconLoader::SizeEnormous, KIconLoader::SizeEnormous);
+        s.boundedTo(size().toSize());
+        painter->drawPixmap(QPoint(0, 0), m_currentWeather.pixmap(s));
+    }
+
+private:
+    KIcon m_currentWeather;
+};
+
 WeatherApplet::WeatherApplet(QObject *parent, const QVariantList &args)
         : WeatherPopupApplet(parent, args),
         m_locationLabel(new Plasma::Label),
         m_forecastTemps(new Plasma::Label),
         m_conditionsLabel(new Plasma::Label),
-        m_currentIcon(0),
         m_tempLabel(new Plasma::Label),
-        m_windIcon(new Plasma::IconWidget),
         m_courtesyLabel(new Plasma::Label),
         m_tabBar(new Plasma::TabBar),
         m_fiveDaysModel(0),
@@ -69,7 +115,7 @@ WeatherApplet::WeatherApplet(QObject *parent, const QVariantList &args)
         m_titleFrame(new Plasma::Frame)
 {
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
-    setPopupIcon("weather-not-available");
+    setPopupIcon("weather-none-available");
 }
 
 QGraphicsWidget *WeatherApplet::graphicsWidget()
@@ -80,7 +126,8 @@ QGraphicsWidget *WeatherApplet::graphicsWidget()
 void WeatherApplet::init()
 {
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(reloadTheme()));
-    m_graphicsWidget = new QGraphicsWidget(this);
+    m_graphicsWidget = new BackgroundWidget(this);
+    connect(this, SIGNAL(newWeatherSource()), this, SLOT(clearCurrentWeatherIcon()));
 
     switch (formFactor()) {
     case Plasma::Horizontal:
@@ -104,7 +151,6 @@ void WeatherApplet::init()
 
     m_titlePanel->setHorizontalSpacing(0);
     m_titlePanel->setVerticalSpacing(0);
-    m_bottomLayout = new QGraphicsLinearLayout(Qt::Horizontal);
 
     m_locationLabel->nativeWidget()->setAlignment(Qt::AlignLeft | Qt::AlignAbsolute);
     m_titleFont = QApplication::font();
@@ -118,10 +164,6 @@ void WeatherApplet::init()
     m_conditionsLabel->nativeWidget()->setWordWrap(false);
     m_conditionsLabel->setMinimumWidth(55);
 
-    m_windIcon->setMaximumSize(0,0);
-    m_windIcon->setOrientation(Qt::Horizontal);
-    m_windIcon->setTextBackgroundColor(QColor());
-
     m_tempLabel->nativeWidget()->setAlignment(Qt::AlignRight | Qt::AlignAbsolute);
     m_tempLabel->nativeWidget()->setFont(m_titleFont);
 
@@ -178,6 +220,11 @@ void WeatherApplet::toolTipAboutToShow()
     Plasma::ToolTipManager::self()->setContent(this, data);
 }
 
+void WeatherApplet::clearCurrentWeatherIcon()
+{
+    m_graphicsWidget->setCurrentWeather(QString());
+}
+
 void WeatherApplet::resizeView()
 {
     if (m_fiveDaysView) {
@@ -270,7 +317,7 @@ void WeatherApplet::setVisibleLayout(bool val)
     }
 
     setVisible(val, m_titlePanel);
-    setVisible(val, m_bottomLayout);
+    m_tabBar->setVisible(val);
 
     m_courtesyLabel->setVisible(val);
 }
@@ -332,42 +379,6 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
         m_tempLabel->setText(i18nc("Not available","N/A"));
     }
 
-    if (!m_currentIcon) {
-        kDebug() << "Create new Plasma::IconWidget (condition)";
-        m_currentIcon = new Plasma::IconWidget(m_graphicsWidget);
-        m_currentIcon->resize(KIconLoader::SizeEnormous, KIconLoader::SizeEnormous);
-        m_currentIcon->setMinimumWidth(KIconLoader::SizeSmall);
-        m_currentIcon->setZValue(900);
-        m_currentIcon->setPos(0,0);
-        //m_currentIcon = new Plasma::IconWidget(KIcon(data["Condition Icon"].toString()), QString(), this);
-        //m_currentIcon->icon().pixmap(QSize(KIconLoader::SizeEnormous,KIconLoader::SizeEnormous));
-        m_currentIcon->setDrawBackground(false);
-        m_currentIcon->setAcceptHoverEvents(false);
-        m_currentIcon->show();
-    }
-
-    if (!m_windIcon) {
-        kDebug() << "Create new Plasma::IconWidget (wind)";
-        m_windIcon = new Plasma::IconWidget();
-        m_windIcon->setOrientation(Qt::Horizontal);
-        m_windIcon->setDrawBackground(false);
-    }
-
-    if (data["Wind Speed"] != "N/A" && data["Wind Speed"].toDouble() != 0 && data["Wind Speed"] != "Calm") {
-        KUnitConversion::Value v(data["Wind Speed"].toDouble(), data["Wind Speed Unit"].toInt());
-        v = v.convertTo(speedUnit());
-        m_windIcon->setText(i18nc("wind direction, speed","%1 %2 %3", data["Wind Direction"].toString(),
-                clampValue(v.number(), 1), v.unit()->symbol()));
-    } else {
-        if (data["Wind Speed"] == "N/A") {
-            m_windIcon->setText(i18nc("Not available","N/A"));
-        } else {
-            if (data["Wind Speed"].toInt() == 0 || data["Wind Speed"] == "Calm") {
-                m_windIcon->setText(i18nc("Wind condition","Calm"));
-            }
-        }
-    }
-
     m_courtesyLabel->setText(data["Credit"].toString());
 
     if (!data["Credit Url"].toString().isEmpty()) {
@@ -383,19 +394,25 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
 
         if (fiveDayTokens.count() > 2) {
             // if there is no specific icon, show the current weather
-            m_currentIcon->setIcon(KIcon(fiveDayTokens[1]));
+            m_graphicsWidget->setCurrentWeather(fiveDayTokens[1]);
+            setPopupIcon(KIcon(fiveDayTokens[1]));
         } else {
             // if we are inside here, we could not find any proper icon
             // then just hide it
-            m_currentIcon->hide();
+            m_graphicsWidget->setCurrentWeather();
+            setPopupIcon(KIcon("weather-none-available"));
         }
     } else {
-        m_currentIcon->setIcon(KIcon(data["Condition Icon"].toString()));
-        m_currentIcon->show();
-        setPopupIcon(data["Condition Icon"].toString());
+        m_graphicsWidget->setCurrentWeather(data["Condition Icon"].toString());
+        const QString condition(data["Condition Icon"].toString());
+        if (isValidIconName(condition)) {
+            setPopupIcon(condition);
+        } else {
+            setPopupIcon("weather-not-available");
+        }
     }
 
-    m_tabBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+    m_tabBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
 
     // If we have items in tab clean it up first
     while (m_tabBar->count()) {
@@ -407,7 +424,6 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
         if (!m_fiveDaysView) {
             kDebug() << "Create 5 Days Plasma::WeatherView";
             m_fiveDaysView = new Plasma::WeatherView(m_tabBar);
-            connect(m_fiveDaysView->nativeWidget()->header(), SIGNAL(sectionResized(int, int, int)), this, SLOT(fiveDaysColumnResized(int, int, int)));
         }
 
         if (!m_fiveDaysModel) {
@@ -454,7 +470,11 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
             // If we see N/U (Not Used) we skip the item
             if (fiveDayTokens[1] != "N/U") {
                 Plasma::IconWidget *fiveDayIcon = new Plasma::IconWidget(this);
-                fiveDayIcon->setIcon(KIcon(fiveDayTokens[1]));
+                if (isValidIconName(fiveDayTokens[1])) {
+                    fiveDayIcon->setIcon(KIcon(fiveDayTokens[1]));
+                } else {
+                    fiveDayIcon->setIcon(KIcon("weather-not-available"));
+                }
                 fiveDayIcon->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
                 fiveDayIcon->setDrawBackground(false);
                 fiveDayIcon->hide();
@@ -603,13 +623,22 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
         Plasma::Svg svgWindIcon;
         svgWindIcon.setImagePath("weather/wind-arrows");
         QIcon windIcon = svgWindIcon.pixmap(data["Wind Direction"].toString());
-        m_windIcon->setIcon(windIcon);
-        m_windIcon->setMaximumSize(m_windIcon->sizeFromIconSize(KIconLoader::SizeSmall));
-        m_windIcon->update();
 
-        QStandardItem *windInfo = new QStandardItem(m_windIcon->icon(), NULL);
+        QString text;
+        if (data["Wind Speed"] != "N/A" && data["Wind Speed"].toDouble() != 0 && data["Wind Speed"] != "Calm") {
+            KUnitConversion::Value v(data["Wind Speed"].toDouble(), data["Wind Speed Unit"].toInt());
+            v = v.convertTo(speedUnit());
+            text = i18nc("wind direction, speed","%1 %2 %3", data["Wind Direction"].toString(),
+                        clampValue(v.number(), 1), v.unit()->symbol());
+        } else if (data["Wind Speed"] == "N/A") {
+                text = i18nc("Not available","N/A");
+        } else if (data["Wind Speed"].toInt() == 0 || data["Wind Speed"] == "Calm") {
+                    text = i18nc("Wind condition","Calm");
+        }
+
+        QStandardItem *windInfo = new QStandardItem(windIcon, NULL);
         windInfo->setTextAlignment(Qt::AlignRight);
-        windInfo->setText(m_windIcon->text());
+        windInfo->setText(text);
         m_detailsModel->appendRow(windInfo);
     }
 
@@ -709,15 +738,7 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
     }
 
     if (!m_setupLayout) {
-        //m_bottomLayout->addItem(m_currentIcon);
-        QGraphicsWidget *spacer = new QGraphicsWidget(this);
-        spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-        spacer->setMinimumWidth(KIconLoader::SizeMedium);
-        spacer->setMaximumWidth(KIconLoader::SizeHuge);
-        m_bottomLayout->addItem(spacer);
-        m_bottomLayout->addItem(m_tabBar);
-        m_bottomLayout->setStretchFactor(m_tabBar, 2);
-        m_layout->addItem(m_bottomLayout);
+        m_layout->addItem(m_tabBar);
         m_layout->addItem(m_courtesyLabel);
         m_setupLayout = 1;
     }
