@@ -70,12 +70,8 @@ Plasma::Service* OcsEngine::serviceForSource(const QString& source)
     if (request == "Person" || request == "Settings") {
         QString id = arguments.value("id");
         QString providerString = arguments.value("provider");
-        if (!id.isEmpty() && !providerString.isEmpty()) {
-            QPair<QString, QString> key = qMakePair(providerString, id);
-            if (!m_personServices.contains(key)) {
-                m_personServices[key] = new PersonService(m_providers.value(providerString), id, m_serviceUpdates, this);
-            }
-            return m_personServices[key];
+        if (!id.isEmpty() && !providerString.isEmpty() && m_providers.value(providerString)) {
+            return new PersonService(m_providers.value(providerString), id, m_serviceUpdates, this);
         }
     }
     return Plasma::DataEngine::serviceForSource(source);
@@ -431,6 +427,9 @@ bool OcsEngine::sourceRequestEvent(const QString& source)
         QString baseUrl = arguments.value("provider");
 
         Provider* provider = m_providers.value(baseUrl).data();
+        if (!provider) {
+            m_sourcesWithoutProvider[baseUrl].append(source);
+        }
         bool success = providerDependentRequest(request, arguments, source, baseUrl, provider);
         if (!success) {
             qDebug() << "Source failed:" << source;
@@ -602,6 +601,7 @@ void OcsEngine::slotKnowledgeBaseListResult(BaseJob* j)
         qDebug() << "Getting knowledgebase list" << source << "failed with code" << j->metadata().statusCode();
     }
     setStatusData(source, j);
+    forceImmediateUpdateOfAllVisualizations();
 }
 
 void OcsEngine::slotPersonListResult(BaseJob* j)
@@ -861,7 +861,6 @@ void OcsEngine::slotMessageListResult(BaseJob* j)
 
 void OcsEngine::providerAdded(const Attica::Provider& provider)
 {
-    
     qDebug() << "providerAdded" << provider.baseUrl();
 
     QString baseUrl = provider.baseUrl().toString();
@@ -869,6 +868,11 @@ void OcsEngine::providerAdded(const Attica::Provider& provider)
         m_providers.insert(baseUrl, QSharedPointer<Provider>(new Provider(provider)));
 
         updateProviderData();
+
+        foreach (QString source, m_sourcesWithoutProvider.value(provider.baseUrl().toString())) {
+            sourceRequestEvent(source);
+        }
+        m_sourcesWithoutProvider.remove(provider.baseUrl().toString());
     }
 }
 
@@ -882,7 +886,7 @@ void OcsEngine::updateProviderData()
         removeData("Providers", "loading");
     }
     
-    foreach(QSharedPointer<Attica::Provider> provider, m_providers) {
+    foreach(const QSharedPointer<Attica::Provider> &provider, m_providers) {
         Plasma::DataEngine::Data providerData;
         
         providerData.insert("BaseUrl", provider->baseUrl());
